const { fork } = require('child_process');
const path = require('path');
const gulp = require('gulp');
const less = require('gulp-less');
const tsc = require.resolve('typescript/bin/tsc');
const rename = require('gulp-rename');
const fs = require('fs-extra');
const copyNodeModules = require('copy-node-modules');
const miniConfig = require('../mini.project');

function compileLess() {
  return gulp.src([
    '../src/**/*.less',
   ])
  .pipe(less())
  .pipe(rename({
    extname: '.acss',
  }))
  .pipe(gulp.dest('../dist'));
}

function compileTS(done) {
  fork(tsc, {
    stdio: 'inherit',
    cwd: '../src',
  });
  done();
}

function copyAssets() {
  return gulp.src([
    '../src/**/**',
    '!/**/*.ts',
    '!/**/*.less',
  ]).pipe(gulp.dest('../dist'));
}

function cleanDist(done) {
  fs.remove('../dist').then(() => {
    done();
  }).catch(err => {
    done(err);
  });
}

function copy(src, target) {
  return new Promise((resolve, reject) => {
    copyNodeModules(src, target, { devDependencies: false }, (err) => {
      if (err) {
        reject(err);
        return;
      }
      resolve()
    });
  })
}

function copyPackages(done) {
  const src = path.join(__dirname, '..');
  const miniprogramRoot = path.join(__dirname, '..', miniConfig.miniprogramRoot);
  const pluginRoot = path.join(__dirname, '..', miniConfig.pluginRoot);
  copy(src, miniprogramRoot).then(() => copy(src, pluginRoot)).then(done).catch(err => done(err));
}

exports.compileLess = compileLess;
exports.clean = cleanDist;

exports.default = gulp.series(cleanDist, gulp.parallel(copyPackages, copyAssets, compileLess, compileTS));
